import numpy as np
import zengl

from window import Window

window = Window(1280, 720)
ctx = zengl.context()

image = ctx.image(window.size, 'rgba8unorm', samples=4)
image.clear_value = (1.0, 1.0, 1.0, 1.0)

uniform_buffer = ctx.buffer(size=16)

triangle = ctx.pipeline(
    vertex_shader='''
        #version 330

        layout (std140) uniform Common {
            vec2 move;
            vec2 scale;
        };

        out vec3 v_color;

        vec2 positions[3] = vec2[](
            vec2(0.0, 0.8),
            vec2(-0.6, -0.8),
            vec2(0.6, -0.8)
        );

        vec3 colors[3] = vec3[](
            vec3(1.0, 0.0, 0.0),
            vec3(0.0, 1.0, 0.0),
            vec3(0.0, 0.0, 1.0)
        );

        void main() {
            gl_Position = vec4(positions[gl_VertexID] * scale + move, 0.0, 1.0);
            v_color = colors[gl_VertexID];
        }
    ''',
    fragment_shader='''
        #version 330

        in vec3 v_color;

        layout (location = 0) out vec4 out_color;

        void main() {
            out_color = vec4(v_color, 1.0);
        }
    ''',
    layout=[
        {
            'name': 'Common',
            'binding': 0,
        },
    ],
    resources=[
        {
            'type': 'uniform_buffer',
            'binding': 0,
            'buffer': uniform_buffer,
        },
    ],
    framebuffer=[image],
    topology='triangles',
    vertex_count=3,
)

while window.update():
    t = window.time
    z = np.frombuffer(uniform_buffer.map(), 'f4')
    z[:] = [
        np.sin(t) * 0.1,
        np.cos(t) * 0.1,
        0.7 + np.sin(t * 5) * 0.05,
        0.7 + np.sin(t * 5) * 0.05,
    ]
    uniform_buffer.unmap()

    image.clear()
    triangle.render()
    image.blit()
